from oscarbot.menu import Menu, Button
from oscarbot.response import TGResponse


def start(user):
    menu = Menu([
        Button("Начнем", callback="/diagnostic/"),
    ])
    return TGResponse(
        message="Привет! Мы здесь, чтобы продиагностировать твой бизнес. Начнем?",
        video='https://file-examples.com/storage/fe5048eb7365a64ba96daa9/2017/04/file_example_MP4_480_1_5MG.mp4'
        # menu=menu
    )


def first_question(user):
    menu = Menu([
        Button("Да", callback="/diagnostic/"),
    ])
    return TGResponse(
        message="Привет! Мы здесь, чтобы продиагностировать твой бизнес. Начнем?",
        menu=menu
    )



from django.apps import apps
from django.conf import settings


def get_bot_model():
    try:
        app_name, app_model = settings.OSCARBOT_BOT_MODEL.split('.')
        bot_model = apps.get_model(app_name, app_model)
        return bot_model
    except Exception:
        raise RuntimeError('Failed to get Bot model')



# Generated by Django 4.2.6 on 2023-10-09 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Bot',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('t_id', models.CharField(default='', max_length=100, verbose_name='Telegram ID')),
                ('token', models.CharField(blank=True, max_length=255, null=True, verbose_name='Bot token')),
                ('name', models.CharField(default='', max_length=250, verbose_name='Name')),
                ('username', models.CharField(default='', max_length=250, verbose_name='Username')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from oscarbot.bot import Bot


class TGResponse:

    def __init__(self, message: str, menu=None, need_update=False, photo=None, attache=None, video=None) -> None:
        # self.chat = chat
        self.message = message
        self.menu = menu
        self.attache = attache
        self.need_update = need_update
        self.photo = photo
        self.video = video

    def send(self, token, user=None, t_id=None):
        tg_bot = Bot(token)
        if self.menu:
            self.menu = self.menu.build()
        data_to_send = {
            'chat_id': user.t_id if user is not None else t_id,
            'message': self.message,
            'reply_keyboard': self.menu,
            'photo': self.photo,
            'video': self.video
        }

        if self.need_update:
            response_content = tg_bot.update_message(
                **data_to_send,
                message_id=user.last_message_id,
            )
        else:
            response_content = tg_bot.send_message(
                **data_to_send
            )
        print(response_content)
        if user:
            user.update_last_sent_message(response_content)

    def can_send(self):
        if self.message is not None:
            return True
        return False



from django.test import TestCase

from oscarbot.models import Bot


class SmokeTestCase(TestCase):

    def setUp(self) -> None:
        self.token = 'token'
        self.bot = Bot.objects.create(
            t_id=1,
            username='oscarbot',
            token=self.token
        )

    def __get_command(self, command):
        command = '''{"update_id": 1,"callback_query": {"id": "1","from": {"id": 310780174,"is_bot": false, ''' \
                  '''"first_name": "Oleg","last_name": "Maslov","username": "maslov_oa","language_code": "en"},''' \
                  '''"message": {"message_id": 1,"from": {"id": 310780174,"is_bot": true,"first_name": ''' \
                  '''"Auto Seller","username": "autoseller_oscar_bot"},"chat": {"id": 310780174,"first_name": ''' \
                  '''"Oleg","last_name": "Maslov","username": "maslov_oa","type": "private"},"date": 1616245487,''' \
                  '''"text": "test","chat_instance": "310780174","data": "''' + command + '''"}}'''
        return command

    def __get_text(self, text):
        text = '''{"update_id": 1,"message": {"message_id": 1,"from": {"id": 310780174,"is_bot": false,''' \
               '''"first_name": "Oleg","last_name": "Maslov","username": "maslov_oa","language_code": "en"},''' \
               '''"chat": {"id": 310780174,"first_name": "Oleg","last_name": "Maslov","username": "maslov_oa",''' \
               '''"type": "1"},"date": 310780174,"text": "''' + text + '''","entities": [{"offset": 0,''' \
               '''"length": 6,"type": "bot_command"}]}}'''
        return text

    def test_smoke_bot(self):
        response = self.client.post(
            f'/api/bot{self.token}/',
            data=self.__get_command('/course/1/'),
            content_type='application/json'
        )

        self.assertEquals(
            response.status_code,
            200
        )



import importlib

import requests
from django.conf import settings

from oscarbot.action import Action
from oscarbot.bot import Bot
from oscarbot.models import User
from oscarbot.response import TGResponse
from oscarbot.router import Router
from oscarbot.structures import Message


class BaseHandler:

    # logger: Logger

    def __init__(self, bot, content):
        self.bot_model = bot
        self.bot = Bot(bot.token)
        self.content = content
        self.message = Message(content)
        self.user = self.__find_or_create_user_in_db()

    def __find_or_create_user_in_db(self):
        if hasattr(self.message, 'user'):
            user_in_db, _ = User.objects.update_or_create(
                t_id=self.message.user.id,
                defaults={
                    "username": self.message.user.username,
                    "name": f'{self.message.user.first_name} {self.message.user.last_name}'
                },
            )
            return user_in_db
        return None

    @staticmethod
    def __send_do_not_understand():
        return TGResponse(
            message="Извините, я не понимаю Вас :("
        )

    def handle(self) -> TGResponse:
        if hasattr(self.message, 'data') and self.message.data:
            return self.__handle_callback_data(self.message.data)
        elif hasattr(self.message, 'text') and self.message.text:
            return self.__handle_text_data()
        elif hasattr(self.message, 'photo') and self.message.photo:
            return self.__handle_photo_data()
        elif hasattr(self.message, 'document') and self.message.document:
            return self.__handle_document_data()
        else:
            return self.__send_do_not_understand()

    def __get_text_handler(self, photo=None):
        mod_name, func_name = settings.TELEGRAM_TEXT_PROCESSOR.rsplit('.', 1)
        mod = importlib.import_module(mod_name)
        text_processor = getattr(mod, func_name)
        data = {
            'text': self.message.text,
            'photo': photo,
        }
        response = text_processor(self.user, data)
        if response:
            return response

        return False

    def __handle_callback_data(self, path):
        router = Router(path)
        func, arguments = router()
        if func:
            response = func(user=self.user, **arguments)
            if response:
                return response
        return self.__send_do_not_understand()

    def __handle_text_data(self):
        if self.message.text[0] == '/':
            return self.__handle_callback_data(self.message.text)

        if self.user.want_action:
            action = Action(self.user, self.message.text)
            return action()

        if settings.TELEGRAM_TEXT_PROCESSOR:
            response = self.__get_text_handler()
            if response:
                return response

        return self.__send_do_not_understand()

    def __handle_photo_data(self):
        """ WIP: """
        photos = []
        for file in self.message.photo:
            file_id = file['file_id']
            res = requests.get(f'{settings.TELEGRAM_URL}{self.bot.token}/getFile?file_id={file_id}')
            file_path = res.json()['result']['file_path']
            photos.append(
                f'https://api.telegram.org/file/bot{self.bot.token}/{file_path}'
            )

        if settings.TELEGRAM_TEXT_PROCESSOR:
            response = self.__get_text_handler(photo=photos)
            if response:
                return response

        return self.__send_do_not_understand()

    def __handle_document_data(self):
        """ WIP: """
        return TGResponse(
            message=''
        )



import json
import os
import re

from django.conf import settings
import requests


class Bot:
    token = None
    menu = None
    api_url = settings.TELEGRAM_URL

    def __init__(self, token):
        self.token = token

    def send_message(self, chat_id, message, photo=None, video=None, is_silent=False, is_background=False, reply_to_msg_id=None,
                     parse_mode='HTML', reply_keyboard=None):
        """
        @param chat_id:
        @param message:
        @param is_silent:
        @param is_background:
        @param reply_to_msg_id:
        @param reply_keyboard:
        @param parse_mode:
        @return: reply message json
        :param line_button:
        """
        params = {
            'chat_id': chat_id,
            'text': re.sub('<[^>]*>', '', message),  # TODO: todo something
            'parse_mode': parse_mode,
            'protect_content': True
        }
        if reply_keyboard is not None:
            params['reply_markup'] = reply_keyboard
        if is_silent:
            params['silent'] = True
            params['disable_notification'] = True
        if is_background:
            params['background'] = True
        if reply_to_msg_id:
            params['reply_to_msg_id'] = reply_to_msg_id

        params['disable_web_page_preview'] = True
        if photo:
            params['photo'] = f'{settings.BASE_URL}{settings.MEDIA_URL}{photo}'
            params['caption'] = params['text']
            result = requests.post(self.api_url + self.token + "/sendPhoto", data=params)
        elif video:
            if video.startswith('https://'):
                params['video_note'] = video
            else:
                params['video_note'] = f'{settings.BASE_URL}{settings.MEDIA_URL}{video}'
            params['caption'] = params['text']
            result = requests.post(self.api_url + self.token + "/sendVideoNote", data=params)
        else:
            result = requests.post(self.api_url + self.token + "/sendMessage", data=params)
        content = result.content.decode('utf-8')
        return content

    def send_photo(self, chat_id, photo_url):
        """
        @param chat_id:
        @param photo_url:
        @return: reply message json
        """
        params = {'chat_id': chat_id, 'photo': photo_url, 'parse_mode': 'Markdown'}
        result = requests.post(self.api_url + self.token + "/sendPhoto", data=params)
        content = result.content.decode('utf-8')
        return content

    def send_video(self, chat_id, video_url):
        params = {
            'chat_id': chat_id,
            'video_note': video_url,
            'parse_mode': 'Markdown',
            'length': 100
        }
        result = requests.post(self.api_url + self.token + "/sendVideoNote", data=params)
        content = result.content.decode('utf-8')
        return content

    def send_photos(self, chat_id, photos):
        photo_group = []
        for photo in photos:
            photo_group.append(
                {
                    'type': 'photo',
                    'media': photo
                }
            )
        params = {'chat_id': chat_id, 'media': json.dumps(photo_group)}
        result = requests.post(self.api_url + self.token + "/sendMediaGroup", data=params)
        content = result.content.decode('utf-8')
        return content

    def update_message(self, chat_id, message_id, message, photo=None, reply_keyboard=None):
        params = {
            'chat_id': chat_id,
            'message_id': message_id,
            'text': re.sub('<[^>]*>', '', message),  # TODO: todo something
            'parse_mode': 'HTML'
        }
        if reply_keyboard is not None:
            params['reply_markup'] = reply_keyboard
        result = requests.post(self.api_url + self.token + "/editMessageText", data=params)
        return result.content.decode('utf-8')

    def delete_message(self, chat_id, message_id):
        params = {'chat_id': chat_id, 'message_id': message_id}
        response = requests.post(self.api_url + self.token + '/deleteMessage', data=params)
        return response

    def get_file(self, file_id):
        result = requests.get(self.api_url + self.token + f'/getFile?file_id={file_id}')
        json_result = result.json()
        if json_result['ok']:
            file_path = json_result['result']['file_path']

            return f"https://api.telegram.org/file/bot{self.token}/{file_path}"
        return None

    def get_user_photo(self, t_id):
        result = requests.get(self.api_url + self.token + '/getUserProfilePhotos', data={'user_id': t_id})
        json_result = result.json()
        if json_result['ok']:
            photos = json_result['result']['photos']
            if len(photos) > 0:
                smallest_photo = photos[0][0]['file_id']
                result = requests.get(self.api_url + self.token + f'/getFile?file_id={smallest_photo}')
                json_result = result.json()
                if json_result['ok']:
                    photo_path = json_result['result']['file_path']
                    result = requests.get(f"https://api.telegram.org/file/bot{self.token}/{photo_path}")
                    media_file = f'users/{t_id}.jpg'
                    with open(os.path.join(settings.BASE_DIR, f'media/{media_file}'), 'wb') as f:
                        f.write(result.content)
                    return media_file
        return None
